==> download.php?file=Net2Plan-0.3.0.zip: com/net2plan/interfaces/simulation/SimEvent.java <==
/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.interfaces.simulation;

import com.net2plan.utils.StringUtils;
import java.util.Collections;
import java.util.EnumSet;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>Class representing a simulation event.</p>
 *
 * <p>Regardless specific event details, every event is defined by the event arrival time and a priority value. The highest priority event is called first.</p>
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.0
 * @see <a href="http://en.wikipedia.org/wiki/Discrete_event_simulation">Wikipedia, "Discrete event simulation," <i>Wikipedia</i>, <i>The Free Encyclopedia</i></a>
 */
public class SimEvent implements Comparable<SimEvent>
{
	/**
	 * Module that will receive the associated event.
	 * 
	 * @since 0.3.0
	 */
	public enum DestinationModule
	{
		/**
		 * Event generator.
		 *
		 * @since 0.3.0
		 */
		EVENT_GENERATOR("event generator"),

		/**
		 * Event processor.
		 *
		 * @since 0.3.0
		 */
		EVENT_PROCESSOR("event processor");
	
		private final String label;
		private DestinationModule(String label) { this.label = label; }
		
		@Override
		public String toString() { return label; }
	};
	
	/**
	 * Default priority for events (higher values give higher priority).
	 * 
	 * @since 0.2.0
	 */
	public final static int DEFAULT_PRIORITY = 0;
	
	/**
	 * Default type for events.
	 * 
	 * @since 0.3.0
	 */
	public final static int DEFAULT_TYPE = 0;
	
	private final double eventTime;
	private int priority;
	private final Object object;
	private final List<SimAction> actions;
	private final DestinationModule destinationModule;
	private final EnumSet<SimAction.ActionType> actionTypes;
	private int type;
	
	/**
	 * Constructor that allows to generate an event with a custom object.
	 *
	 * @param eventTime Event time
	 * @param object Custom object
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, Object object, DestinationModule destinationModule)
	{
		this(eventTime, null, object, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with an action notification.
	 *
	 * @param eventTime Event time
	 * @param action Action to be notified
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, final SimAction action, DestinationModule destinationModule)
	{
		this(eventTime, new LinkedList<SimAction>() {{ add(action); }}, null, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with a list of action notifications.
	 *
	 * @param eventTime Event time
	 * @param actions List of actions to be notified
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, List<SimAction> actions, DestinationModule destinationModule)
	{
		this(eventTime, actions, null, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with a custom object and a list of action notifications.
	 * 
	 * @param eventTime Event time
	 * @param actions List of actions to be notified
	 * @param object Custom object
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, List<SimAction> actions, Object object, DestinationModule destinationModule)
	{
		this.eventTime = eventTime;
		this.actions = new LinkedList<SimAction>();
		this.actionTypes = EnumSet.noneOf(SimAction.ActionType.class);
		this.object = object;
		this.destinationModule = destinationModule;
		
		if (actions != null)
		{
			for(SimAction action : actions)
			{
				this.actions.add(action);
				this.actionTypes.add(action.getActionType());
			}
		}
		
		setEventPriority(DEFAULT_PRIORITY);
		setEventType(DEFAULT_TYPE);
	}

	@Override
	public final int compareTo(SimEvent e)
	{
		/* Compare event time */
		if (getEventTime() < e.getEventTime())
		{
			return -1;
		}
		else
		{
			if (getEventTime() > e.getEventTime())
			{
				return 1;
			}
			else
			{
				/* If equal, sort by priority */
				if (getEventPriority() < e.getEventPriority()) return 1;
				else return getEventPriority() > e.getEventPriority() ? -1 : 0;
			}
		}
	}

	@Override
	public String toString()
	{
		String NEW_LINE = StringUtils.getLineSeparator();
		StringBuilder out = new StringBuilder();
		out.append(String.format("New event for %s at %s (priority = %d, type = %d)", destinationModule.label, StringUtils.secondsToYearsDaysHoursMinutesSeconds(eventTime), priority, type));
		out.append(NEW_LINE).append(NEW_LINE);
		
		if (actions.isEmpty())
		{
			out.append("No action specified");
			out.append(NEW_LINE);
		}
		else
		{
			out.append("List of notified actions:");
			out.append(NEW_LINE);
			
			for(SimAction action : actions)
			{
				out.append(action.toString());
				out.append(NEW_LINE);
			}
		}
		
		if (object == null)
		{
			out.append("No custom object specified");
			out.append(NEW_LINE);
		}
		else
		{
			out.append("Custom object: ").append(object.toString());
			out.append(NEW_LINE);
		}
		
		return out.toString();
	}
	
	/**
	 * Returns <code>true</code> if the event contains an action notification of the
	 * given type.
	 *
	 * @param actionType Type of action notification
	 * @return <code>true</code> if the event contains an action notification of the given type. Otherwise, <code>false</code>
	 * @since 0.3.0
	 */
	public boolean contains(SimAction.ActionType actionType)
	{
		return containsAny(EnumSet.of(actionType));
	}
	
	/**
	 * Returns <code>true</code> if the event contains an action notification of 
	 * among one of the given types.
	 *
	 * @param actionTypes Types of action notification
	 * @return <code>true</code> if the event contains an action notification of any of the given type. Otherwise, <code>false</code>
	 * @since 0.3.0
	 */
	public boolean containsAny(EnumSet<SimAction.ActionType> actionTypes)
	{
		for(SimAction.ActionType actionType : actionTypes)
			if (this.actionTypes.contains(actionType))
				return true;
		
		return false;
	}
	
	/**
	 * <p>Returns the list of action notifications.</p>
	 *
	 * <p><b>Important</b>: The returned list is unmodifiable</p>
	 *
	 * @return List of action notifications
	 * @since 0.3.0
	 */
	public final List<SimAction> getEventActionList()
	{
		return Collections.unmodifiableList(actions);
	}
	
	/**
	 * <p>Returns the destination module of the event.</p>
	 *
	 * @return List of action notifications
	 * @since 0.3.0
	 */
	public final DestinationModule getEventDestinationModule()
	{
		return destinationModule;
	}
	
	/**
	 * Returns the custom object of the event (may be null).
	 * 
	 * @return Custom object
	 * @since 0.3.0
	 */
	public final Object getEventObject()
	{
		return object;
	}
	
	/**
	 * Returns the event priority.
	 * 
	 * @return Event priority
	 * @since 0.2.0
	 */
	public final int getEventPriority()
	{
		return priority;
	}
	
	/**
	 * Returns the event time.
	 * 
	 * @return Event time
	 * @since 0.2.0
	 */
	public final double getEventTime()
	{
		return eventTime;
	}
	
	/**
	 * Returns the custom event type.
	 * 
	 * @return Event type
	 * @since 0.3.0
	 */
	public final int getEventType()
	{
		return type;
	}
	
	/**
	 * Sets scheduling priority of the message. The priority value is used when 
	 * the simulator inserts messages into the future event list (FEL), to order 
	 * messages with identical arrival time values.
	 * 
	 * @param priority Event priority (the higher, the more priority)
	 * @return Reference to the event
	 * @since 0.3.0
	 */
	public final SimEvent setEventPriority(int priority)
	{
		this.priority = priority;
		return this;
	}

	/**
	 * Sets the message kind. This field can be freely used by the user. No 
	 * processing is made by the simulation kernel.
	 * 
	 * @param type Event type
	 * @return Reference to the event
	 * @since 0.3.0
	 */
	public final SimEvent setEventType(int type)
	{
		this.type = type;
		return this;
	}
}

==> download.php?file=Net2Plan-0.3.1.zip: com/net2plan/interfaces/simulation/SimEvent.java <==
/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.interfaces.simulation;

import com.net2plan.utils.StringUtils;
import java.util.Collections;
import java.util.EnumSet;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>Class representing a simulation event.</p>
 *
 * <p>Regardless specific event details, every event is defined by the event arrival time and a priority value. The highest priority event is called first.</p>
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @since 0.2.0
 * @see <a href="http://en.wikipedia.org/wiki/Discrete_event_simulation">Wikipedia, "Discrete event simulation," <i>Wikipedia</i>, <i>The Free Encyclopedia</i></a>
 */
public class SimEvent implements Comparable<SimEvent>
{
	/**
	 * Module that will receive the associated event.
	 * 
	 * @since 0.3.0
	 */
	public enum DestinationModule
	{
		/**
		 * Event generator.
		 *
		 * @since 0.3.0
		 */
		EVENT_GENERATOR("event generator"),

		/**
		 * Event processor.
		 *
		 * @since 0.3.0
		 */
		EVENT_PROCESSOR("event processor");
	
		private final String label;
		private DestinationModule(String label) { this.label = label; }
		
		@Override
		public String toString() { return label; }
	};
	
	/**
	 * Default priority for events (higher values give higher priority).
	 * 
	 * @since 0.2.0
	 */
	public final static int DEFAULT_PRIORITY = 0;
	
	/**
	 * Default type for events.
	 * 
	 * @since 0.3.0
	 */
	public final static int DEFAULT_TYPE = 0;
	
	private final double eventTime;
	private int priority;
	private final Object object;
	private final List<SimAction> actions;
	private final DestinationModule destinationModule;
	private final EnumSet<SimAction.ActionType> actionTypes;
	private int type;
	
	/**
	 * Constructor that allows to generate an event with a custom object.
	 *
	 * @param eventTime Event time
	 * @param object Custom object
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, Object object, DestinationModule destinationModule)
	{
		this(eventTime, null, object, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with an action notification.
	 *
	 * @param eventTime Event time
	 * @param action Action to be notified
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, final SimAction action, DestinationModule destinationModule)
	{
		this(eventTime, new LinkedList<SimAction>() {{ add(action); }}, null, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with a list of action notifications.
	 *
	 * @param eventTime Event time
	 * @param actions List of actions to be notified
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, List<SimAction> actions, DestinationModule destinationModule)
	{
		this(eventTime, actions, null, destinationModule);
	}
	
	/**
	 * Constructor that allows to generate an event with a custom object and a list of action notifications.
	 * 
	 * @param eventTime Event time
	 * @param actions List of actions to be notified
	 * @param object Custom object
	 * @param destinationModule Module that will receive the event
	 * @since 0.3.0
	 */
	public SimEvent(double eventTime, List<SimAction> actions, Object object, DestinationModule destinationModule)
	{
		this.eventTime = eventTime;
		this.actions = new LinkedList<SimAction>();
		this.actionTypes = EnumSet.noneOf(SimAction.ActionType.class);
		this.object = object;
		this.destinationModule = destinationModule;
		
		if (actions != null)
		{
			for(SimAction action : actions)
			{
				this.actions.add(action);
				this.actionTypes.add(action.getActionType());
			}
		}
		
		setEventPriority(DEFAULT_PRIORITY);
		setEventType(DEFAULT_TYPE);
	}

	@Override
	public final int compareTo(SimEvent e)
	{
		/* Compare event time */
		if (getEventTime() < e.getEventTime())
		{
			return -1;
		}
		else
		{
			if (getEventTime() > e.getEventTime())
			{
				return 1;
			}
			else
			{
				/* If equal, sort by priority */
				if (getEventPriority() < e.getEventPriority()) return 1;
				else return getEventPriority() > e.getEventPriority() ? -1 : 0;
			}
		}
	}

	@Override
	public String toString()
	{
		String NEW_LINE = StringUtils.getLineSeparator();
		StringBuilder out = new StringBuilder();
		out.append(String.format("New event for %s at %s (priority = %d, type = %d)", destinationModule.label, StringUtils.secondsToYearsDaysHoursMinutesSeconds(eventTime), priority, type));
		out.append(NEW_LINE).append(NEW_LINE);
		
		if (actions.isEmpty())
		{
			out.append("No action specified");
			out.append(NEW_LINE);
		}
		else
		{
			out.append("List of notified actions:");
			out.append(NEW_LINE);
			
			for(SimAction action : actions)
			{
				out.append(action.toString());
				out.append(NEW_LINE);
			}
		}
		
		if (object == null)
		{
			out.append("No custom object specified");
			out.append(NEW_LINE);
		}
		else
		{
			out.append("Custom object: ").append(object.toString());
			out.append(NEW_LINE);
		}
		
		return out.toString();
	}
	
	/**
	 * Returns {@code true} if the event contains an action notification of the
	 * given type.
	 *
	 * @param actionType Type of action notification
	 * @return {@code true} if the event contains an action notification of the given type. Otherwise, {@code false}
	 * @since 0.3.0
	 */
	public boolean contains(SimAction.ActionType actionType)
	{
		return containsAny(EnumSet.of(actionType));
	}
	
	/**
	 * Returns {@code true} if the event contains an action notification of 
	 * among one of the given types.
	 *
	 * @param actionTypes Types of action notification
	 * @return {@code true} if the event contains an action notification of any of the given type. Otherwise, {@code false}
	 * @since 0.3.0
	 */
	public boolean containsAny(EnumSet<SimAction.ActionType> actionTypes)
	{
		for(SimAction.ActionType actionType : actionTypes)
			if (this.actionTypes.contains(actionType))
				return true;
		
		return false;
	}
	
	/**
	 * <p>Returns the list of action notifications.</p>
	 *
	 * <p><b>Important</b>: The returned list is unmodifiable</p>
	 *
	 * @return List of action notifications
	 * @since 0.3.0
	 */
	public final List<SimAction> getEventActionList()
	{
		return Collections.unmodifiableList(actions);
	}
	
	/**
	 * <p>Returns the destination module of the event.</p>
	 *
	 * @return List of action notifications
	 * @since 0.3.0
	 */
	public final DestinationModule getEventDestinationModule()
	{
		return destinationModule;
	}
	
	/**
	 * Returns the custom object of the event (may be null).
	 * 
	 * @return Custom object
	 * @since 0.3.0
	 */
	public final Object getEventObject()
	{
		return object;
	}
	
	/**
	 * Returns the event priority.
	 * 
	 * @return Event priority
	 * @since 0.2.0
	 */
	public final int getEventPriority()
	{
		return priority;
	}
	
	/**
	 * Returns the event time.
	 * 
	 * @return Event time
	 * @since 0.2.0
	 */
	public final double getEventTime()
	{
		return eventTime;
	}
	
	/**
	 * Returns the custom event type.
	 * 
	 * @return Event type
	 * @since 0.3.0
	 */
	public final int getEventType()
	{
		return type;
	}
	
	/**
	 * Sets scheduling priority of the message. The priority value is used when 
	 * the simulator inserts messages into the future event list (FEL), to order 
	 * messages with identical arrival time values.
	 * 
	 * @param priority Event priority (the higher, the more priority)
	 * @return Reference to the event
	 * @since 0.3.0
	 */
	public final SimEvent setEventPriority(int priority)
	{
		this.priority = priority;
		return this;
	}

	/**
	 * Sets the message kind. This field can be freely used by the user. No 
	 * processing is made by the simulation kernel.
	 * 
	 * @param type Event type
	 * @return Reference to the event
	 * @since 0.3.0
	 */
	public final SimEvent setEventType(int type)
	{
		this.type = type;
		return this;
	}
}
